@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import argparse
+import os
 import sys
 import traceback
 try:
@@ -14,6 +15,7 @@ from libearth.repository import from_url
 from libearth.schema import SchemaError
 from libearth.session import Session
 from libearth.stage import Stage
+from sassutils.wsgi import SassMiddleware
 from waitress import serve
 
 from .app import app, spawn_worker
@@ -30,13 +32,22 @@ def crawl_command(args):
     if not opml:
         print('OPML does not exist in the repository', file=sys.stderr)
         return
-    urllist = [subscription.feed_uri
-               for subscription in opml.recursive_subscriptions]
-    id_map = dict((sub.feed_uri, sub.feed_id)
-                  for sub in opml.recursive_subscriptions)
+    feed_id = args.feed_id
+    if feed_id:
+        feed_map = dict((sub.feed_uri, sub.feed_id)
+                        for sub in opml.recursive_subscriptions
+                        if sub.feed_id == feed_id)
+        if not feed_map:
+            print('There is no such feed:', feed_id, file=sys.stderr)
+            return
+    else:
+        feed_map = dict((sub.feed_uri, sub.feed_id)
+                        for sub in opml.recursive_subscriptions)
+        if not feed_map:
+            print('No feeds to crawl', file=sys.stderr)
+            return
     threads_count = args.threads if args.threads is not None else cpu_count()
-
-    iterator = iter(crawl(urllist, threads_count))
+    iterator = iter(crawl(feed_map.keys(), threads_count))
     while 1:
         try:
             feed_url, feed_data, crawler_hints = next(iterator)
@@ -45,7 +56,7 @@ def crawl_command(args):
                     feed_data, len(feed_data.entries)
                 ))
             with stage:
-                feed_id = id_map[feed_url]
+                feed_id = feed_map[feed_url]
                 stage.feeds[feed_id] = feed_data
         except (CrawlError, SchemaError) as e:
             print('Something went wrong with', feed_url, file=sys.stderr)
@@ -76,13 +87,16 @@ def server_command(args):
         app.wsgi_app = make_linesman_middleware(app.wsgi_app)
     spawn_worker()
     if args.debug:
+        app.wsgi_app = SassMiddleware(app.wsgi_app, {
+            'earthreader.web': ('static/scss/', 'static/css/')
+        })
         app.run(host=args.host, port=args.port, debug=args.debug, threaded=True)
     else:
         serve(app, host=args.host, port=args.port)
 
 
 parser = argparse.ArgumentParser(prog='earthreader')
-subparsers = parser.add_subparsers(dest='command', help='command-help')
+subparsers = parser.add_subparsers(dest='command')
 
 server_parser = subparsers.add_parser('server',
                                       help='run a server for Earth Reader')
@@ -119,6 +133,9 @@ crawl_parser.add_argument('-i', '--session-id',
                           help='session identifier.  [default: %(default)s]')
 crawl_parser.add_argument('-v', '--verbose', default=False, action='store_true',
                           help='verbose mode')
+crawl_parser.add_argument('-f', '--feed-id',
+                          help='crawl only the specified feed.  '
+                               'crawl all subscriptions by default')
 crawl_parser.add_argument('repository', help='repository which has the opml')
 
 
@@ -130,7 +147,8 @@ def main():
 
     url = urlparse.urlparse(args.repository)
     if url.scheme == '':
-        args.repository = urlparse.urljoin('file://', args.repository)
+        args.repository = urlparse.urljoin(
+            'file://', os.path.join(os.getcwd(), args.repository))
 
     args.function(args)
 
