@@ -83,6 +83,10 @@ function unreadCurrent() {
 	var url = current.attr('data-read-url');
 	var method = 'DELETE';
 
+	if (!url) {
+		return;
+	}
+
 	$.ajax(url, {
 		'type': method,
 	}).done(function() {
@@ -97,6 +101,10 @@ function toggleStarCurrent() {
 	var url = current.attr('data-star-url');
 	var method;
 
+	if (!url) {
+		return;
+	}
+
 	if (current.length === 0) {
 		return;
 	}
@@ -193,7 +201,6 @@ function processForm(event) {
 var makeCategory = function(parentObj, obj) {
 	var container = $('<li>');
 	var header = $('<div>');
-	var list = $('<ul>');
 	var i;
 
 	header.addClass('feed header');
@@ -217,6 +224,9 @@ var makeCategory = function(parentObj, obj) {
 	if (obj.path){
 		container.attr('data-path', obj.path);
 	}
+	if (obj.feeds_url) {
+		container.attr('data-feed-url', obj.feeds_url);
+	}
 
 	header.html(obj.title.link(obj.entries_url));
 
@@ -228,23 +238,14 @@ var makeCategory = function(parentObj, obj) {
 	handle.addClass('handle');
 	header.prepend(handle);
 
+	header.addClass('closed');
 	header.attr('draggable', true);
 
-	$.get(obj.feeds_url, function(obj) {
-		for (i=0; i<obj.categories.length; i++) {
-			makeCategory(list, obj.categories[i]);
-		}
-		for (i=0; i<obj.feeds.length; i++) {
-			makeFeed(list, obj.feeds[i]);
-		}
-	});
-	list.addClass('fold');
 	header.on('dragstart', dragStart);
 	header.on('dragover', dragOver);
 	header.on('drop', drop);
 
 	container.append(header);
-	container.append(list);
 
 	container.addClass('folder');
 	parentObj.append(container);
@@ -272,7 +273,7 @@ var makeFeed = function(parentObj, obj) {
 function makeFeedList(obj, target) {
 	var feedList;
 	var i;
-  
+
 	if (target && target.length !== 0) {
 		feedList = target;
 	} else {
@@ -295,11 +296,9 @@ function refreshFeedList() {
 }
 
 function dragStart(event) {
-	if (!$(event.target).is('li')) {
-		event.target = $(event.target).closest('li');
-	}
-	var target = $(event.target).attr('data-path');
-	event.originalEvent.dataTransfer.setData('data-path', target);
+	var target = $(event.target).closest('li');
+	var dst = $(event.target).attr('data-path');
+	event.originalEvent.dataTransfer.setData('data-path', dst);
 }
 
 function dragOver(event) {
@@ -322,7 +321,7 @@ function dropToHeader(event) {
 function moveOutline(url) {
 	$.ajax({
 		url: url,
-		type: 'put'
+	type: 'put'
 	}).done(function(){
 		refreshFeedList();
 	}).fail(printError);
@@ -338,7 +337,7 @@ function getAllEntries() {
 
 function appendEntry(entry) {
 	var main = $('[role=main]');
-	
+
 	var article = $('<article>');
 	var header = $('<div>');
 	var time = $('<time>');
@@ -382,55 +381,54 @@ function appendEntry(entry) {
 	main.append(article);
 }
 
-function getEntries(feed_url, filter) {
+function processEntries(obj, queryUrl) {
 	var main = $('[role=main]');
-	var requestUrl = feed_url;
-	if (filter) {
-		requestUrl += '?' + filter;
+	var feed_title = obj.title;
+	var entries = obj.entries;
+	var header = $('<header>');
+	var h2 = $('<h2>');
+	var refresh = $('<a>');
+	var mark_all = $('<a>');
+
+	main.html(null);
+
+	h2.text(feed_title);
+	header.append(h2);
+
+	refresh.addClass('refresh');
+	refresh.attr('role', 'button');
+	refresh.attr('href', queryUrl);
+	refresh.text('crawl now');
+	header.append(refresh);
+
+	mark_all.addClass('mark_all');
+	mark_all.attr('href', obj.read_url);
+	mark_all.text('mark all as read');
+	header.append(mark_all);
+
+	main.append(header);
+
+	if (entries.length === 0) {
+		var noEntry = $('<p>');
+		noEntry.text('No more entry');
+		main.append(noEntry);
+	} else {
+		for (var i=0; i<entries.length; i++) {
+			appendEntry(entries[i]);
+		}
 	}
 
-	$.get(requestUrl, function(obj) {
-		var feed_title = obj.title;
-		var entries = obj.entries;
-		var header = $('<header>');
-		var h2 = $('<h2>');
-		var refresh = $('<a>');
-
-		main.html(null);
-
-		h2.text(feed_title);
-		header.append(h2);
-
-		refresh.addClass('refresh');
-		refresh.attr('role', 'button');
-		refresh.attr('href', feed_url);
-		refresh.text('crawl now');
-		header.append(refresh);
-
-		main.append(header);
-
-		if (entries.length === 0) {
-			var noEntry = $('<p>');
-			noEntry.text('No more entry');
-			main.append(noEntry);
-		} else {
-			for (var i=0; i<entries.length; i++) {
-				appendEntry(entries[i]);
-			}
+	if (obj.next_url) {
+		var nextLoader = main.find('.nextPage');
+		if (nextLoader.length === 0) {
+			nextLoader = $('<div>');
+			nextLoader.addClass('nextPage');
+			nextLoader.text("Load next page");
 		}
-
-		if (obj.next_url) {
-			var nextLoader = main.find('.nextPage');
-			if (nextLoader.length === 0) {
-				nextLoader = $('<div>');
-				nextLoader.addClass('nextPage');
-				nextLoader.text("Load next page");
-			}
-			nextLoader.attr('data-next-url', obj.next_url);
-			main.append(nextLoader);
-		}
-		$(window).scrollTop(0);
-	}).fail(printError);
+		nextLoader.attr('data-next-url', obj.next_url);
+		main.append(nextLoader);
+	}
+	$(window).scrollTop(0);
 }
 
 function loadNextPage() {
@@ -484,59 +482,115 @@ function reloadEntries() {
 	var filter = currentFilter.attr('data-filter');
 	var url = currentFeed.attr('data-entries') || currentFeed.parent().attr('data-entries') || URLS.entries;
 
-	getEntries(url, filter);
-}
-
-function clickFeed(event) {
-	var target = $(event.target);
-	var feedlist = $('[role=navigation] .feedlist');
-
-	while (target.hasClass('feed') === false) {
-		//toggle folding
-		if (target.hasClass('toggle')) {
-			target.parent().toggleClass('closed');
-			return;
-		}
-		target = target.parent();
+	if (filter) {
+		url += '?' + filter;
 	}
 
-	//set current marker
-	feedlist.find('.current').removeClass('current');
-	target.addClass('current');
+	$('[role=navigation] .loading').removeClass('loading');
+	currentFeed.addClass('loading');
 
+	$.get(url, function (obj) {
+		currentFeed.removeClass('loading');
+		if (currentFeed.hasClass('current')) {
+			processEntries(obj, url);
+		}
+	}).fail(printError);
+}
+
+function loadSubCategory(container) {
+	var list = $('<ul>');
+	container = container.closest('.folder');
+	$.get(container.attr('data-feed-url'), function(obj) {
+		for (i=0; i<obj.categories.length; i++) {
+			makeCategory(list, obj.categories[i]);
+		}
+		for (i=0; i<obj.feeds.length; i++) {
+			makeFeed(list, obj.feeds[i]);
+		}
+	});
+	list.addClass('fold');
+	container.append(list);
+}
+
+function selectFeed(feed) {
+	var feedlist = $('[role=navigation] .feedlist');
+	feedlist.find('.current').removeClass('current');
+	feed.closest('.feed').addClass('current');
+}
+
+function enterFeed(feed) {
+	selectFeed(feed);
+
+	if (feed.hasClass('closed')) {
+		feed.removeClass('closed');
+		if (feed.siblings('.fold').length === 0) {
+			loadSubCategory(feed);
+		}
+	}
 
 	closeMenu();
 
 	reloadEntries();
 }
 
-function clickEntry(event) {
-	var target = $(event.target).closest('.entry-title');
+function clickFeed(event) {
+	var target = $(event.target);
+	var feedlist = $('[role=navigation] .feedlist');
+	var feed = target.closest('.feed');
+
+	//set current marker
+	selectFeed(feed);
+
+	if (target.closest('.toggle').length) {
+		feed.toggleClass('closed');
+		if (feed.siblings('.fold').length === 0) {
+			loadSubCategory(feed);
+		}
+		return;
+	}
+
+	enterFeed(feed);
+}
+
+function selectEntry(entry) {
 	var main = $('[role=main]');
 
-	var entry = target.parent();
-	var entry_url = entry.attr('data-entries');
+	entry = $(entry).closest('.entry');
+	main.find('.current').removeClass('current');
+	entry.addClass('current');
+	$(window).scrollTop(entry.position().top);
+}
 
+function toggleEntryCollapse(entry) {
+	var entry_url;
+	var content;
+	var entry_title;
+	var main = $('[role=main]');
 
-	//close content
-	var content = entry.find('.entry-content');
+	entry = entry.closest('.entry');
+	entry_url = entry.attr('data-entries');
+	entry_title = entry.find('.entry-title');
+	content = entry.find('.entry-content');
+
+	// close content
 	if (content.length) {
 		content.remove();
 		return;
 	}
 
+	main.find('.entry.loading').removeClass('loading');
+	entry.addClass('loading');
+
 	$.get(entry_url, function(obj) {
 		var i;
-		//set current marker
-		main.find('.current').removeClass('current');
-		target.parent().addClass('current');
-
 		//remove content
 		contents = main.find('.entry-content');
 		for (i=0; i<contents.length; i++) {
 			contents[i].parentElement.removeChild(contents[i]);
 		}
 
+		entry.removeClass('loading');
+
 		var wrapper = $('<div>');
 		var title = $('<h1>');
 		var content = $('<div>');
@@ -558,20 +612,28 @@ function clickEntry(event) {
 		wrapper.append(bottom_bar);
 		entry.append(wrapper);
 
-		target.parent().attr('data-read-url', obj.read_url);
-		target.parent().attr('data-star-url', obj.star_url);
+		entry.attr('data-read-url', obj.read_url);
+		entry.attr('data-star-url', obj.star_url);
 		//read
 		$.ajax({
 			'type': 'put',
 			'url': obj.read_url,
 		}).done(function() {
-			target.addClass('read');
+			entry_title.addClass('read');
 		});
 
-		$(window).scrollTop(entry.position().top);
+		//set current marker
+		selectEntry(entry);
+
 	}).fail(printError);
 }
 
+function clickEntry(event) {
+	var entry = $(event.target).closest('.entry');
+
+	toggleEntryCollapse(entry);
+}
+
 function refreshFeed() {
 	var target = $('[role=main] .refresh');
 	var url = target.attr('href');
@@ -582,6 +644,18 @@ function refreshFeed() {
 		});
 }
 
+function markAllRead() {
+	var target = $('[role=main] .mark_all');
+	var url = target.attr('href');
+
+	target.addClass('requesting');
+	$.ajax(url, {'type': 'PUT'})
+		.done(function() {
+			target.removeClass('requesting');
+			$('.entry-title').addClass('read');
+		});
+}
+
 function clickLink(event) {
 	var target = event.target;
 
@@ -599,37 +673,58 @@ function keyboardShortcut(event) {
 		return;
 	}
 
+	if (event.ctrlKey || event.metaKey) {
+		return;
+	}
+
 	var main = $('[role=main]');
 	var feedlist = $('[role=navigation] .feedlist');
 
-	var nextEntry = function nextEntry() {
+	var nextEntry = function nextEntry(open) {
+		var first;
 		var entry = main.find('.current').first();
 		if (entry.length === 0) {
-			main.find('.entry .entry-title').first().click();
+			first = main.find('.entry').first();
+			if (first.length === 0) {
+				return;
+			}
+			selectEntry(first);
+			if (open) {
+				toggleEntryCollapse(first);
+			}
 			return;
 		}
 		var next = entry.next().first();
 		if (next.length === 0) {
 			return;
 		}
-		next.find('.entry-title').click();
+
+		if (open) {
+			toggleEntryCollapse(next);
+		} else {
+			selectEntry(next);
+		}
 	};
 
-	var prevEntry = function prevEntry() {
+	var prevEntry = function prevEntry(open) {
 		var entry = main.find('.current').first();
 		if (entry.length === 0) {
 			return;
 		}
 		var prev = entry.prev().last();
-		if (prev.length === 0 || prev.hasClass('entry')) {
-			//close current
-			entry.click();
+		if (prev.length === 0 || prev.hasClass('entry') === false) {
+			return;
+		}
+
+		if (open) {
+			toggleEntryCollapse(prev);
+		} else {
+			selectEntry(prev);
 		}
-		prev.find('.entry-title').click();
 	};
 
 	var nextFeed = function nextFeed() {
-		var feeds = feedlist.find('.feed');
+		var feeds = feedlist.find('.feed:visible');
 		if (feeds.filter('.current').length === 0) {
 			feeds.first().click();
 			return;
@@ -643,7 +738,7 @@ function keyboardShortcut(event) {
 	};
 
 	var prevFeed = function prevFeed() {
-		var feeds = feedlist.find('.feed');
+		var feeds = feedlist.find('.feed:visible');
 
 		if (feeds.first().hasClass('current')) {
 			feedlist.find('.allfeed.header').click();
@@ -665,31 +760,42 @@ function keyboardShortcut(event) {
 		}
 	};
 
-	var closeEntry = function closeEntry() {
-		main.find('article.current .entry-content').remove();
+	var toggleEntry = function closeEntry() {
+		var entry = main.find('.current');
+		toggleEntryCollapse(entry);
+	};
+
+	var toggleFolding = function toggleFolding() {
+		feedlist.find('.current').children('.toggle').click();
 	};
 
 	switch (event.keyCode) {
 		case 13: // return
-			closeEntry();
+		case 79:
+			toggleEntry();
 			break;
 		case 27: // esc
 			closeManual();
 			break;
 		case 74: //j
-			nextEntry();
+			if (event.shiftKey) {
+				nextFeed();
+			} else {
+				nextEntry(true);
+			}
 			break;
 		case 75: //k
-			prevEntry();
+			if (event.shiftKey) {
+				prevFeed();
+			} else {
+				prevEntry(true);
+			}
 			break;
 		case 78: //n
-			nextFeed();
-			break;
-		case 79: //o
-			openInNewTab();
+			nextEntry();
 			break;
 		case 80: //p
-			prevFeed();
+			prevEntry();
 			break;
 		case 82: //r
 			reloadEntries();
@@ -698,9 +804,17 @@ function keyboardShortcut(event) {
 			toggleStarCurrent();
 			break;
 		case 85: //u
-        case 77: //m
+		case 77: //m
 			unreadCurrent();
 			break;
+		case 86: //v
+			openInNewTab();
+			break;
+		case 88: // x
+			if (event.shiftKey) {
+				toggleFolding();
+			}
+			break;
 		case 191: // /
 			// ?
 			if (event.shiftKey) {
@@ -748,6 +862,7 @@ $(function () {
 	main.on('click', '.entry-title', clickEntry);
 	main.on('click', '.nextPage', loadNextPage);
 	main.on('click', '.refresh', refreshFeed);
+	main.on('click', '.mark_all', markAllRead);
 
 	var side = $('[role=complementary]');
 	side.on('click', '[data-action]', clickComplementaryMenu);
@@ -763,19 +878,19 @@ $(function () {
 	$('#manual').click(closeManual);
 
 	//FIXME: clean it
-    var animationEnd;
-    if (document.body.style.animation !== undefined) {
-        animationEnd = 'animationend';
-    } else if (document.body.style.MozAnimation !== undefined) {
-        animationEnd = 'animaionend';
-    } else if (document.body.style.webkitAnimation !== undefined) {
-        animationEnd = 'webkitAnimationEnd';
-    } else if (document.body.style.OAnimtion !== undefined) {
-        animationEnd = 'oAnimationEnd';
-    } else if (document.body.style.msAnimation !== undefined) {
-        animationEnd = 'MSAnimaionEnd';
-    }
-    $(document.body).on(animationEnd, resizer);
+	var animationEnd;
+	if (document.body.style.animation !== undefined) {
+		animationEnd = 'animationend';
+	} else if (document.body.style.MozAnimation !== undefined) {
+		animationEnd = 'animaionend';
+	} else if (document.body.style.webkitAnimation !== undefined) {
+		animationEnd = 'webkitAnimationEnd';
+	} else if (document.body.style.OAnimtion !== undefined) {
+		animationEnd = 'oAnimationEnd';
+	} else if (document.body.style.msAnimation !== undefined) {
+		animationEnd = 'MSAnimaionEnd';
+	}
+	$(document.body).on(animationEnd, resizer);
 
 	$('[role=navigation] .persistent .unread').addClass('current');
 	$('[role=navigation] .feedlist .allfeed.header').addClass('current');
@@ -789,3 +904,5 @@ $(function () {
 		.delay('slow')
 		.fadeIn('fast');
 });
+
+/* vim: noet:ts=4 */
